<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="ganttview_scale" version="4" translationDomain="calligraplanlibs">
    <MenuBar>
        <Menu name="view">
            <Action name="show_project"/>
            <Action name="show_unscheduled_tasks"/>
            <Separator/>
            <Action name="expand_all"/>
            <Action name="collapse_all"/>
            <Separator/>
            <Menu name="gantt">
                <text>Gantt</text>
                <Action name="scale_auto"/>
                <Action name="scale_month"/>
                <Action name="scale_week"/>
                <Action name="scale_day"/>
                <Action name="scale_hour"/>
                <Separator/>
                <Action name="zoom_in"/>
                <Action name="zoom_out"/>
            </Menu>
            <Separator/>
            <Action name="print"/>
            <Action name="print_preview"/>
            <Action name="print_pdf"/>
            <Action name="print_options"/>
            <Separator/>
            <Action name="configure_view"/>
        </Menu>
    </MenuBar>
    <Menu name="gantt_datetimegrid_popup">
        <text>Gantt</text>
        <Action name="scale_auto"/>
        <Action name="scale_month"/>
        <Action name="scale_week"/>
        <Action name="scale_day"/>
        <Action name="scale_hour"/>
        <Separator/>
        <Action name="zoom_in"/>
        <Action name="zoom_out"/>
    </Menu>
</gui>
