<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="CalligraPlan" version="28" translationDomain="calligraplan">
<MenuBar>
 <Menu name="file" noMerge="1"><text>&amp;File</text>
  <Action name="file_new" />
  <Action name="activeauthor" />
  <Action name="file_open" />
  <Action name="file_open_recent" />
  <Separator />
  <Action name="file_save" />
  <Action name="file_save_as" />
  <Action name="file_reload_file" />
  <Action name="file_versions_file"/>
  <Separator />
  <Action name="file_import_file" />
  <Action name="file_export_file" />
  <Action name="file_send_file" />
  <Separator />
  <Merge/>
  <Separator />
  <Action name="file_print" />
  <Action name="file_print_preview" />
  <Action name="file_export_pdf" />
  <Separator/>
  <Action name="file_documentinfo" />
  <DefineGroup name="file_infos" />
  <Separator />
  <Action name="file_close" />
  <Action name="file_quit" />
 </Menu>
 <Merge />
 <Menu name="settings" noMerge="1"><text>&amp;Settings</text>
  <DefineGroup name="settings_top"/>
  <Merge/>
<!--  <ActionList name="toolbarlist"/> -->
  <Action name="view_toggledockertitlebars"/>
  <Action name="settings_dockers_menu"/>
  <DefineGroup name="settings_show"/>
  <Separator/>
  <Action name="options_configure_keybinding"/>
  <Action name="options_configure_toolbars"/>
  <Action name="theme_menu" />
  <DefineGroup name="settings_configure"/>
  <Action name="configure"/>
 </Menu>
</MenuBar>
<ToolBar name="mainToolBar" fullWidth="false" noMerge="1"><Text>File</Text>
  <Action name="file_new" />
  <Action name="file_open" />
  <Action name="file_save" />
  <!--
  <Action name="file_print" />
  <Action name="file_print_preview" />
  //-->
</ToolBar>
</gui>
