<!DOCTYPE gui SYSTEM "kpartgui.dtd" >
<gui name="PlanWork" version="5" translationDomain="calligraplanwork">
    <MenuBar>
        <Menu name="edit"><text>&amp;Edit</text>
           <Action name="package_remove_current"/>
        </Menu>
    </MenuBar>
    <ToolBar name="view_selection" position="left"><text>View</text>
        <Action name="view_list"/>
        <Action name="view_gantt"/>
    </ToolBar>
    <Menu name="taskstatus_popup">
        <Action name="task_progress"/>
        <Action name="task_description"/>
        <Action name="edit_sendpackage"/>
        <Action name="package_remove_current"/>
    </Menu>
    <Menu name="editdocument_popup">
        <Action name="view_document"/>
        <Action name="edit_document"/>
        <Action name="remove_document"/>
    </Menu>
    <Menu name="viewdocument_popup">
        <Action name="view_document"/>
        <Action name="remove_document"/>
    </Menu>

</gui>
