<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="Calligra Plan" version="5" translationDomain="calligraplan">
<MenuBar>
    <Menu name="view"><text>&amp;View</text>
        <Action name="view_newview"/>
        <Separator/>
    </Menu>
</MenuBar>
</gui>
