<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="KPlatoPluginScripting" library="krossmoduleplan" version="1" translationDomain="krossmoduleplan">
<MenuBar>
    <Menu name="tools"><text>&amp;Tools</text>
    <Separator/>
    <Action name="executescriptfile"/>
    <Action name="scripts"/>
    <Action name="scriptmanager"/>
    </Menu>
</MenuBar>
</gui>
